/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the Source EULA. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
'use strict';
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const vscode = require("vscode");
const path = require("path");
const dataprotocol_client_1 = require("dataprotocol-client");
const service_downloader_1 = require("service-downloader");
const vscode_languageclient_1 = require("vscode-languageclient");
const Constants = require("./constants");
const contextProvider_1 = require("./contextProvider");
const credentialstore_1 = require("./credentialstore/credentialstore");
const resourceProvider_1 = require("./resourceProvider/resourceProvider");
const Utils = require("./utils");
const telemetry_1 = require("./telemetry");
const features_1 = require("./features");
const baseConfig = require('./config.json');
const outputChannel = vscode.window.createOutputChannel(Constants.serviceName);
const statusView = vscode.window.createStatusBarItem(vscode.StatusBarAlignment.Left);
function activate(context) {
    return __awaiter(this, void 0, void 0, function* () {
        // lets make sure we support this platform first
        let supported = yield Utils.verifyPlatform();
        if (!supported) {
            vscode.window.showErrorMessage('Unsupported platform');
            return;
        }
        let config = JSON.parse(JSON.stringify(baseConfig));
        config.installDirectory = path.join(__dirname, config.installDirectory);
        config.proxy = vscode.workspace.getConfiguration('http').get('proxy');
        config.strictSSL = vscode.workspace.getConfiguration('http').get('proxyStrictSSL') || true;
        const credentialsStore = new credentialstore_1.CredentialStore(config);
        const resourceProvider = new resourceProvider_1.AzureResourceProvider(config);
        let languageClient;
        const serverdownloader = new service_downloader_1.ServerProvider(config);
        serverdownloader.eventEmitter.onAny(generateHandleServerProviderEvent());
        let clientOptions = {
            documentSelector: ['sql'],
            synchronize: {
                configurationSection: Constants.extensionConfigSectionName
            },
            providerId: Constants.providerId,
            errorHandler: new telemetry_1.LanguageClientErrorHandler(),
            features: [
                // we only want to add new features
                ...dataprotocol_client_1.SqlOpsDataClient.defaultFeatures,
                features_1.TelemetryFeature,
                features_1.AgentServicesFeature
            ],
            outputChannel: new CustomOutputChannel()
        };
        const installationStart = Date.now();
        serverdownloader.getOrDownloadServer().then(e => {
            const installationComplete = Date.now();
            let serverOptions = generateServerOptions(e);
            languageClient = new dataprotocol_client_1.SqlOpsDataClient(Constants.serviceName, serverOptions, clientOptions);
            const processStart = Date.now();
            languageClient.onReady().then(() => {
                const processEnd = Date.now();
                statusView.text = 'Service Started';
                setTimeout(() => {
                    statusView.hide();
                }, 1500);
                telemetry_1.Telemetry.sendTelemetryEvent('startup/LanguageClientStarted', {
                    installationTime: String(installationComplete - installationStart),
                    processStartupTime: String(processEnd - processStart),
                    totalTime: String(processEnd - installationStart),
                    beginningTimestamp: String(installationStart)
                });
            });
            statusView.show();
            statusView.text = 'Starting service';
            languageClient.start();
            credentialsStore.start();
            resourceProvider.start();
        }, e => {
            telemetry_1.Telemetry.sendTelemetryEvent('ServiceInitializingFailed');
            vscode.window.showErrorMessage('Failed to start Sql tools service');
        });
        let contextProvider = new contextProvider_1.default();
        context.subscriptions.push(contextProvider);
        context.subscriptions.push(credentialsStore);
        context.subscriptions.push(resourceProvider);
        context.subscriptions.push({ dispose: () => languageClient.stop() });
    });
}
exports.activate = activate;
function generateServerOptions(executablePath) {
    let launchArgs = [];
    launchArgs.push('--log-dir');
    let logFileLocation = path.join(Utils.getDefaultLogLocation(), 'mssql');
    launchArgs.push(logFileLocation);
    let config = vscode.workspace.getConfiguration(Constants.extensionConfigSectionName);
    if (config) {
        let logDebugInfo = config[Constants.configLogDebugInfo];
        if (logDebugInfo) {
            launchArgs.push('--enable-logging');
        }
    }
    return { command: executablePath, args: launchArgs, transport: vscode_languageclient_1.TransportKind.stdio };
}
function generateHandleServerProviderEvent() {
    let dots = 0;
    return (e, ...args) => {
        outputChannel.show();
        statusView.show();
        switch (e) {
            case "install_start" /* INSTALL_START */:
                outputChannel.appendLine(`Installing ${Constants.serviceName} service to ${args[0]}`);
                statusView.text = 'Installing Service';
                break;
            case "install_end" /* INSTALL_END */:
                outputChannel.appendLine('Installed');
                break;
            case "download_start" /* DOWNLOAD_START */:
                outputChannel.appendLine(`Downloading ${args[0]}`);
                outputChannel.append(`(${Math.ceil(args[1] / 1024)} KB)`);
                statusView.text = 'Downloading Service';
                break;
            case "download_progress" /* DOWNLOAD_PROGRESS */:
                let newDots = Math.ceil(args[0] / 5);
                if (newDots > dots) {
                    outputChannel.append('.'.repeat(newDots - dots));
                    dots = newDots;
                }
                break;
            case "download_end" /* DOWNLOAD_END */:
                outputChannel.appendLine('Done!');
                break;
        }
    };
}
// this method is called when your extension is deactivated
function deactivate() {
}
exports.deactivate = deactivate;
class CustomOutputChannel {
    append(value) {
        console.log(value);
    }
    appendLine(value) {
        console.log(value);
    }
    clear() {
    }
    show(column, preserveFocus) {
    }
    hide() {
    }
    dispose() {
    }
}
//# sourceMappingURL=https://ticino.blob.core.windows.net/sourcemaps/7099922c35bc3b3e223228c972cc03cee5abbe25/extensions/mssql/out/main.js.map
